// CENGINEDEFINITIONS.H

#if !defined ( _CENGINEDEFINITIONS_H_ )

#define _CENGINEDEFINITIONS_H_

#include <sys/timeb.h>	// ftime()
#include <afxmt.h>		// Mutex


// #define DAMAGE_COORDINATE_OLD
#define DAMAGE_COORDINATE_NEW

enum EngineID { Damage, Horizon3, Horizon4 } ;

enum SearchID { IA, A, N } ;

// 64Bit Definitions
#define DBINDEX		__int64
#define BITBOARD	unsigned __int64
#define HASHKEY		unsigned __int64
#define HASHCODE	unsigned __int64
#define HASHDATA	unsigned __int64
#define INDEX		unsigned __int64
#define COUNT		unsigned __int64

#define ENGINE_ID_DAMAGE				0
#define ENGINE_ID_HORIZON3				1
#define ENGINE_ID_HORIZON4				2

#define SEARCH_STR_SEARCHINFO				"searchinfo"
#define SEARCH_STR_SEARCHINFO_SEARCH		"search"
#define SEARCH_STR_SEARCHINFO_IASEARCH		"iasearch"
#define SEARCH_STR_SEARCHINFO_PV			"pv"
#define SEARCH_STR_SEARCHINFO_STRING		"string"

#define WHITE           1
#define BLACK           0

#define WMAN			0
#define WKING			1
#define BMAN			2
#define BKING			3

#define WHITEMAN		1
#define WHITEKING		2
#define BLACKMAN		-1
#define BLACKKING		-2
#define EMPTY			0

#define TURN            1
#define GAME            2

#define CLEAR           0
#define INIT            1
#define FIELD           2

#define INVALID         -1
#define VALID           0
#define AMBIGIOUS       1

#define COUNT_MOVEGEN				0
#define COUNT_MOVEGEN_FULL			1
#define COUNT_MOVEGEN_KILL			2
#define COUNT_MOVEGEN_NONE			4
#define COUNT_MOVEGEN_MOVE			5
#define COUNT_MOVEGEN_QSEARCH		6
#define COUNT_MOVEGEN_SSEARCH		7

#define COUNT_SEARCH				10
#define COUNT_SEARCH_HASH_CUT		11
#define COUNT_SEARCH_HASH_MOVE		12
#define COUNT_SEARCH_HASH_MOVE_CUT	13
#define COUNT_SEARCH_HASH_MOVE_ERR	14
#define COUNT_SEARCH_HASH_INDEX_ERR	15
#define COUNT_SEARCH_HASH_INDMO_ERR	16
#define COUNT_SEARCH_HASH_ETC_CUT	17
#define COUNT_SEARCH_SINGULARNODE	18
#define COUNT_SEARCH_WRONGCOMB0		19
#define COUNT_SEARCH_WRONGCOMB1		20

#define COUNT_SEARCH_MCP			24
#define COUNT_SEARCH_MCP_SUCCES		25
#define COUNT_SEARCH_MCP_FAIL		26

#define COUNT_SEARCH_MINWIN_FAIL	27
#define COUNT_SEARCH_MINWIN_FAILNOT	28
#define COUNT_SEARCH_MINWIN_FNNODES	29

#define COUNT_SEARCH_FHR1			31
#define COUNT_SEARCH_FHR2			32
#define COUNT_SEARCH_FHR3			33

#define COUNT_SEARCH_BT_OK			34
#define COUNT_SEARCH_BT_ERROR		35

#define COUNT_QSEARCH				40
#define COUNT_SSEARCH				41

#define COUNT_EVALUATION			50
#define COUNT_EVALUATION_EXACT		51
#define COUNT_EVALUATION_SOFT		52
#define COUNT_EVALUATION_MATBALANCE	53
#define COUNT_EVALUATION_MIDSCORE	54
#define COUNT_EVALUATION_HASHREAD	55

#define COUNT_BADMOVE				60
#define COUNT_OUTPUTBOARD			61		// General counter for Output Board !

// MoveGen
#define MOVEGEN_CAPTURE_NOCAPTURE		0
#define MOVEGEN_CAPTURE_WHITEMAN		1	
#define MOVEGEN_CAPTURE_WHITEKING		2
#define MOVEGEN_CAPTURE_BLACKMAN		4	
#define MOVEGEN_CAPTURE_BLACKKING		8

// Search
#define SEARCH_MAXSEARCHDEPTH			32

// General ( Conversion ) Arrays

const int iBallot[316] = { 
	
					31,26,20,25, 31,26,20,24, 31,26,19,24, 31,26,19,23, 31,26,18,23, 31,26,18,22, 31,26,17,22, 31,26,17,21, 31,26,16,21,
					31,27,20,25, 31,27,20,24, 31,27,19,24, 31,27,19,23, 31,27,18,23, 31,27,18,22, 31,27,17,22, 31,27,17,21,
					32,27,20,25, 32,27,20,24, 32,27,19,24, 32,27,19,23, 32,27,18,23, 32,27,18,22, 32,27,17,22, 32,27,17,21,
					32,28,20,25, 32,28,20,24, 32,28,19,24, 32,28,19,23, 32,28,18,23, 32,28,18,22, 32,28,17,22, 32,28,17,21, 32,28,16,21,
					33,28,20,25, 33,28,20,24, 33,28,19,24, 33,28,19,23, 33,28,18,23, 33,28,18,22, 33,28,17,22, 33,28,17,21, 33,28,16,21,
					33,29,20,25, 33,29,20,24, 33,29,19,24, 33,29,19,23, 33,29,18,23, 33,29,18,22, 33,29,17,22, 33,29,17,21, 33,29,16,21,
					34,29,20,25, 34,29,20,24, 34,29,19,24, 34,29,19,23, 34,29,18,23, 34,29,18,22, 34,29,17,22, 34,29,17,21, 34,29,16,21,
					34,30,20,25, 34,30,20,24, 34,30,19,24, 34,30,19,23, 34,30,18,23, 34,30,18,22, 34,30,17,22, 34,30,17,21, 34,30,16,21,
					35,30,20,25, 35,30,20,24, 35,30,19,24, 35,30,19,23, 35,30,18,23, 35,30,18,22, 35,30,17,22, 35,30,17,21, 35,30,16,21 } ;

const int iInitBoard[51] = {	0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
								0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
								1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 } ;

const int SLW[50] = {
    0, 0, 0, 0, 0,
    40, 40,40, 40 , 40,
    35, 35, 35, 35, 35,
    30, 30, 30, 30, 30,
    25, 25, 25, 25, 25,
    20, 20, 20, 20, 20,
    15, 15, 15, 15, 15,
    10, 10, 10 ,10, 10,
    5, 5, 5, 5, 5,
    0, 0, 0, 0 ,0
} ;

const int SLB[50] = {
    0, 0, 0, 0, 0,
    5, 5, 5, 5, 5,
    10, 10, 10, 10, 10,
    15, 15, 15, 15, 15,
    20, 20, 20, 20, 20,
    25, 25, 25, 25 ,25,
    30, 30, 30, 30, 30,
    35, 35, 35, 35, 35,
    40, 40, 40, 40, 40,
    0 ,0, 0, 0, 0
} ;

#ifdef DAMAGE_COORDINATE_OLD

//Define DFLD BITBOARDS
#define DFLD1	0x200000000000000LL
#define DFLD2	0x400000000000000LL
#define DFLD3	0x800000000000000LL
#define DFLD4	0x1000000000000000LL
#define DFLD5	0x2000000000000000LL
#define DFLD6	0x8000000000000LL
#define DFLD7	0x10000000000000LL
#define DFLD8	0x20000000000000LL
#define DFLD9	0x40000000000000LL
#define DFLD10	0x80000000000000LL
#define DFLD11	0x400000000000LL
#define DFLD12	0x800000000000LL
#define DFLD13	0x1000000000000LL
#define DFLD14	0x2000000000000LL
#define DFLD15	0x4000000000000LL
#define DFLD16	0x10000000000LL
#define DFLD17	0x20000000000LL
#define DFLD18	0x40000000000LL
#define DFLD19	0x80000000000LL
#define DFLD20	0x100000000000LL
#define DFLD21	0x800000000LL
#define DFLD22	0x1000000000LL
#define DFLD23	0x2000000000LL
#define DFLD24	0x4000000000LL
#define DFLD25	0x8000000000LL
#define DFLD26	0x20000000LL
#define DFLD27	0x40000000LL
#define DFLD28	0x80000000LL
#define DFLD29	0x100000000LL
#define DFLD30	0x200000000LL
#define DFLD31	0x1000000LL
#define DFLD32	0x2000000LL
#define DFLD33	0x4000000LL
#define DFLD34	0x8000000LL
#define DFLD35	0x10000000LL
#define DFLD36	0x40000LL
#define DFLD37	0x80000LL
#define DFLD38	0x100000LL
#define DFLD39	0x200000LL
#define DFLD40	0x400000LL
#define DFLD41	0x2000LL
#define DFLD42	0x4000LL
#define DFLD43	0x8000LL
#define DFLD44	0x10000LL
#define DFLD45	0x20000LL
#define DFLD46	0x80LL
#define DFLD47	0x100LL
#define DFLD48	0x200LL
#define DFLD49	0x400LL
#define DFLD50	0x800LL

// Directions..
#define DIRECTION_FL	5
#define DIRECTION_FR	6
#define DIRECTION_BL	-6
#define DIRECTION_BR	-5

// Inline Functions 

inline BITBOARD DFLBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 1 ) ; }


inline BITBOARD DBRFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 1 ) ; }


inline BITBOARD DFLFLBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 4 ) ; }


inline BITBOARD DBRBRFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 4 ) ; }


inline BITBOARD DFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 5 ) ; }


inline BITBOARD DBR(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 5 ) ; }


inline BITBOARD DFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 6 ) ; }


inline BITBOARD DBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 6 ) ; }


inline BITBOARD DFRFRBR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 7 ) ; }


inline BITBOARD DBLBLFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 7 ) ; }


inline BITBOARD DFLFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 10 ) ; }


inline BITBOARD DBRBR(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 10 ) ; }


inline BITBOARD DFRFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 11 ) ; }


inline BITBOARD DBRBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 11 ) ; }


inline BITBOARD DFRFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 12 ) ; }


inline BITBOARD DBLBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 12 ) ; }


inline BITBOARD DFLFLFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 15 ) ; }


inline BITBOARD DBRBRBR(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 15 ) ; }


inline BITBOARD DFLFLFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 16 ) ; }


inline BITBOARD DBRBRBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 16 ) ; }


inline BITBOARD DFRFRFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 17 ) ; }


inline BITBOARD DBLBLBR(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 17 ) ; }


inline BITBOARD DFRFRFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 18 ) ; }


inline BITBOARD DBLBLBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 18 ) ; }

#endif	// DAMAGE_COORDINATE_OLD

#ifdef DAMAGE_COORDINATE_NEW

//Define DFLD BITBOARDS
#define DFLD0 0x0ULL
#define DFLD1 0x1ULL
#define DFLD2 0x2ULL
#define DFLD3 0x4ULL
#define DFLD4 0x8ULL
#define DFLD5 0x10ULL
#define DFLD6 0x20ULL
#define DFLD7 0x40ULL
#define DFLD8 0x80ULL
#define DFLD9 0x100ULL
#define DFLD10 0x200ULL
#define DFLD11 0x800ULL
#define DFLD12 0x1000ULL
#define DFLD13 0x2000ULL
#define DFLD14 0x4000ULL
#define DFLD15 0x8000ULL
#define DFLD16 0x10000ULL
#define DFLD17 0x20000ULL
#define DFLD18 0x40000ULL
#define DFLD19 0x80000ULL
#define DFLD20 0x100000ULL
#define DFLD21 0x400000ULL
#define DFLD22 0x800000ULL
#define DFLD23 0x1000000ULL
#define DFLD24 0x2000000ULL
#define DFLD25 0x4000000ULL
#define DFLD26 0x8000000ULL
#define DFLD27 0x10000000ULL
#define DFLD28 0x20000000ULL
#define DFLD29 0x40000000ULL
#define DFLD30 0x80000000ULL
#define DFLD31 0x200000000ULL
#define DFLD32 0x400000000ULL
#define DFLD33 0x800000000ULL
#define DFLD34 0x1000000000ULL
#define DFLD35 0x2000000000ULL
#define DFLD36 0x4000000000ULL
#define DFLD37 0x8000000000ULL
#define DFLD38 0x10000000000ULL
#define DFLD39 0x20000000000ULL
#define DFLD40 0x40000000000ULL
#define DFLD41 0x100000000000ULL
#define DFLD42 0x200000000000ULL
#define DFLD43 0x400000000000ULL
#define DFLD44 0x800000000000ULL
#define DFLD45 0x1000000000000ULL
#define DFLD46 0x2000000000000ULL
#define DFLD47 0x4000000000000ULL
#define DFLD48 0x8000000000000ULL
#define DFLD49 0x10000000000000ULL
#define DFLD50 0x20000000000000ULL
						
// Directions..
#define DIRECTION_FL	-6
#define DIRECTION_FR	-5
#define DIRECTION_BL	5
#define DIRECTION_BR	6

// Inline Functions 

inline BITBOARD DFLBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 1 ) ; }


inline BITBOARD DBRFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 1 ) ; }


inline BITBOARD DFLFLBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 7 ) ; }


inline BITBOARD DBRBRFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 7 ) ; }


inline BITBOARD DFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 6 ) ; }


inline BITBOARD DBR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 6 ) ; }


inline BITBOARD DFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 5 ) ; }


inline BITBOARD DBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 5 ) ; }


inline BITBOARD DFRFRBR(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 4 ) ; }


inline BITBOARD DBLBLFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 4 ) ; }


inline BITBOARD DFLFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 12 ) ; }


inline BITBOARD DBRBR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 12 ) ; }


inline BITBOARD DFRFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 11 ) ; }


inline BITBOARD DFLFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 11 ) ; }


inline BITBOARD DBRBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 11 ) ; }


inline BITBOARD DFRFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 10 ) ; }


inline BITBOARD DBLBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 10 ) ; }


inline BITBOARD DFLFLFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 18 ) ; }


inline BITBOARD DBRBRBR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 18 ) ; }


inline BITBOARD DFLFLFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 17 ) ; }


inline BITBOARD DBRBRBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 17 ) ; }


inline BITBOARD DFRFRFL(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 16 ) ; }


inline BITBOARD DBLBLBR(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 16 ) ; }


inline BITBOARD DFRFRFR(BITBOARD bbBitBoard)
{ return ( bbBitBoard >> 15 ) ; }


inline BITBOARD DBLBLBL(BITBOARD bbBitBoard)
{ return ( bbBitBoard << 15 ) ; }

#endif	// DAMAGE_COORDINATE_NEW

const BITBOARD bbDFLD[51] = {	NULL,
								DFLD1,  DFLD2,  DFLD3,  DFLD4,  DFLD5,  DFLD6,  DFLD7,  DFLD8,  DFLD9,  DFLD10,
								DFLD11, DFLD12, DFLD13, DFLD14, DFLD15, DFLD16, DFLD17, DFLD18, DFLD19, DFLD20,
								DFLD21, DFLD22, DFLD23, DFLD24, DFLD25, DFLD26, DFLD27, DFLD28, DFLD29, DFLD30,
								DFLD31, DFLD32, DFLD33, DFLD34, DFLD35, DFLD36, DFLD37, DFLD38, DFLD39, DFLD40,
								DFLD41, DFLD42, DFLD43, DFLD44, DFLD45, DFLD46, DFLD47, DFLD48, DFLD49, DFLD50
};

const BITBOARD bbNBWhite[128] = {
// White Turn
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( DFLD1 | DFLD2 ),
								( DFLD2 | DFLD3 ),
								( DFLD3 | DFLD4 ),
								( DFLD4 | DFLD5 ),
								( NULL ),
								( DFLD2 | DFLD6 ),
								( DFLD1 | DFLD3 ),
								( DFLD2 | DFLD4 ),
								( DFLD3 | DFLD5 ),
								( NULL ),
								( DFLD2 ),
								( DFLD1 | DFLD2 | DFLD3 | DFLD6 | DFLD8 ),
								( DFLD1 | DFLD2 | DFLD3 | DFLD4 | DFLD7 | DFLD9 ),
								( DFLD2 | DFLD3 | DFLD4 | DFLD5 | DFLD8 | DFLD10 ),
								( DFLD3 | DFLD5 | DFLD15 ),
								( NULL ),
								( DFLD3 | DFLD6 | DFLD8 | DFLD16 ),
								( DFLD1 | DFLD3 | DFLD4 | DFLD6 | DFLD7 | DFLD8 | DFLD9 | DFLD11 | DFLD13 ),
								( DFLD1 | DFLD2 | DFLD4 | DFLD5 | DFLD7 | DFLD8 | DFLD9 | DFLD10 | DFLD12 | DFLD14 ),
								( DFLD2 | DFLD5 | DFLD8 | DFLD10 | DFLD13 | DFLD15 ),
								( NULL ),
								( DFLD3 ),
								( DFLD1 | DFLD3 | DFLD4 | DFLD6 | DFLD9 | DFLD11 | DFLD13 | DFLD16 | DFLD18 ),
								( DFLD1 | DFLD2 | DFLD3 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD9 | DFLD10 | DFLD11 | DFLD12 | DFLD13 | DFLD14 | DFLD17 | DFLD19 ),
								( DFLD1 | DFLD2 | DFLD4 | DFLD5 | DFLD7 | DFLD8 | DFLD10 | DFLD12 | DFLD13 | DFLD14 | DFLD15 | DFLD18 | DFLD20 ),
								( DFLD2 | DFLD5 | DFLD8 | DFLD13 | DFLD15 | DFLD25 ),
								( NULL ),
								( DFLD4 | DFLD6 | DFLD9 | DFLD13 | DFLD16 | DFLD18 | DFLD26 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD9 | DFLD10 | DFLD11 | DFLD13 | DFLD14 | DFLD16 | DFLD17 | DFLD18 | DFLD19 | DFLD21 | DFLD23 ),
								( DFLD1 | DFLD2 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD8 | DFLD9 | DFLD10 | DFLD11 | DFLD12 | DFLD14 | DFLD15 | DFLD17 | DFLD18 | DFLD19 | DFLD20 | DFLD22 | DFLD24 ),
								( DFLD1 | DFLD2 | DFLD5 | DFLD7 | DFLD8 | DFLD10 | DFLD12 | DFLD15 | DFLD18 | DFLD20 | DFLD23 | DFLD25 ),
								( DFLD2 | DFLD8 ),
								( DFLD4 | DFLD9 | DFLD13 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD9 | DFLD10 | DFLD11 | DFLD13 | DFLD14 | DFLD16 | DFLD19 | DFLD21 | DFLD23 | DFLD26 | DFLD28 ),
								( DFLD1 | DFLD2 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD9 | DFLD10 | DFLD11 | DFLD12 | DFLD13 | DFLD14 | DFLD15 | DFLD16 | DFLD17 | DFLD19 | DFLD20 | DFLD21 | DFLD22 | DFLD23 | DFLD24 | DFLD27 | DFLD29 ),
								( DFLD1 | DFLD2 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD10 | DFLD11 | DFLD12 | DFLD14 | DFLD15 | DFLD17 | DFLD18 | DFLD20 | DFLD22 | DFLD23 | DFLD24 | DFLD25 | DFLD28 | DFLD30 ),
								( DFLD1 | DFLD2 | DFLD5 | DFLD7 | DFLD12 | DFLD15 | DFLD18 | DFLD23 | DFLD25 | DFLD35 ),
								( NULL ),
								( DFLD4 | DFLD5 | DFLD6 | DFLD9 | DFLD10 | DFLD14 | DFLD16 | DFLD19 | DFLD23 | DFLD26 | DFLD28 | DFLD36 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD9 | DFLD10 | DFLD11 | DFLD14 | DFLD15 | DFLD16 | DFLD17 | DFLD19 | DFLD20 | DFLD21 | DFLD23 | DFLD24 | DFLD26 | DFLD27 | DFLD28 | DFLD29 | DFLD31 | DFLD33 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD9 | DFLD10 | DFLD11 | DFLD12 | DFLD14 | DFLD15 | DFLD16 | DFLD17 | DFLD18 | DFLD19 | DFLD20 | DFLD21 | DFLD22 | DFLD24 | DFLD25 | DFLD27 | DFLD28 | DFLD29 | DFLD30 | DFLD32 | DFLD34 ),
								( DFLD1 | DFLD5 | DFLD6 | DFLD7 | DFLD10 | DFLD11 | DFLD12 | DFLD15 | DFLD17 | DFLD18 | DFLD20 | DFLD22 | DFLD25 | DFLD28 | DFLD30 | DFLD33 | DFLD35 ),
								( DFLD1 | DFLD7 | DFLD12 | DFLD18 ),
								( DFLD4 | DFLD5 | DFLD10 | DFLD14 | DFLD19 | DFLD23 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD10 | DFLD11 | DFLD14 | DFLD15 | DFLD16 | DFLD19 | DFLD20 | DFLD21 | DFLD23 | DFLD24 | DFLD26 | DFLD29 | DFLD31 | DFLD33 | DFLD36 | DFLD38 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD10 | DFLD11 | DFLD12 | DFLD14 | DFLD15 | DFLD16 | DFLD17 | DFLD19 | DFLD20 | DFLD21 | DFLD22 | DFLD23 | DFLD24 | DFLD25 | DFLD26 | DFLD27 | DFLD29 | DFLD30 | DFLD31 | DFLD32 | DFLD33 | DFLD34 | DFLD37 | DFLD39 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD10 | DFLD11 | DFLD12 | DFLD14 | DFLD15 | DFLD16 | DFLD17 | DFLD20 | DFLD21 | DFLD22 | DFLD24 | DFLD25 | DFLD27 | DFLD28 | DFLD30 | DFLD32 | DFLD33 | DFLD34 | DFLD35 | DFLD38 | DFLD40 ),
								( DFLD1 | DFLD5 | DFLD6 | DFLD7 | DFLD11 | DFLD12 | DFLD15 | DFLD17 | DFLD22 | DFLD25 | DFLD28 | DFLD33 | DFLD35 | DFLD45 ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
// Black Turn
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( DFLD2 ),
								( DFLD1 | DFLD3 ),
								( DFLD2 | DFLD4 ),
								( DFLD3 | DFLD5 ),
								( NULL ),
								( NULL ),
								( DFLD3 | DFLD6 | DFLD8 ),
								( DFLD1 | DFLD4 | DFLD7 | DFLD9 ),
								( DFLD2 | DFLD5 | DFLD8 | DFLD10 ),
								( NULL ),
								( NULL ),
								( DFLD3 ),
								( DFLD1 | DFLD3 | DFLD4 | DFLD6 | DFLD9 | DFLD11 | DFLD13 ),
								( DFLD1 | DFLD2 | DFLD4 | DFLD5 | DFLD7 | DFLD10 | DFLD12 | DFLD14 ),
								( DFLD2 | DFLD5 | DFLD8 | DFLD13 | DFLD15 ),
								( NULL ),
								( NULL ),
								( DFLD4 | DFLD6 | DFLD9 | DFLD13 | DFLD16 | DFLD18 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD9 | DFLD10 | DFLD11 | DFLD14 | DFLD17 | DFLD19 ),
								( DFLD1 | DFLD2 | DFLD5 | DFLD7 | DFLD8 | DFLD10 | DFLD12 | DFLD15 | DFLD18 | DFLD20 ),
								( DFLD2 | DFLD8 ),
								( NULL ),
								( DFLD4 | DFLD9 | DFLD13 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD9 | DFLD10 | DFLD11 | DFLD13 | DFLD14 | DFLD16 | DFLD19 | DFLD21 | DFLD23 ),
								( DFLD1 | DFLD2 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD10 | DFLD11 | DFLD12 | DFLD14 | DFLD15 | DFLD17 | DFLD20 | DFLD22 | DFLD24 ),
								( DFLD1 | DFLD2 | DFLD5 | DFLD7 | DFLD12 | DFLD15 | DFLD18 | DFLD23 | DFLD25 ),
								( DFLD2 ),
								( DFLD4 | DFLD9 ),
								( DFLD4 | DFLD5 | DFLD6 | DFLD9 | DFLD10 | DFLD14 | DFLD16 | DFLD19 | DFLD23 | DFLD26 | DFLD28 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD9 | DFLD10 | DFLD11 | DFLD14 | DFLD15 | DFLD16 | DFLD17 | DFLD19 | DFLD20 | DFLD21 | DFLD24 | DFLD27 | DFLD29 ),
								( DFLD1 | DFLD5 | DFLD6 | DFLD7 | DFLD10 | DFLD11 | DFLD12 | DFLD15 | DFLD17 | DFLD18 | DFLD20 | DFLD22 | DFLD25 | DFLD28 | DFLD30 ),
								( DFLD1 | DFLD7 | DFLD12 | DFLD18 ),
								( NULL ),
								( DFLD4 | DFLD5 | DFLD10 | DFLD14 | DFLD19 | DFLD23 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD10 | DFLD11 | DFLD14 | DFLD15 | DFLD16 | DFLD19 | DFLD20 | DFLD21 | DFLD23 | DFLD24 | DFLD26 | DFLD29 | DFLD31 | DFLD33 ),
								( DFLD1 | DFLD4 | DFLD5 | DFLD6 | DFLD7 | DFLD10 | DFLD11 | DFLD12 | DFLD14 | DFLD15 | DFLD16 | DFLD17 | DFLD20 | DFLD21 | DFLD22 | DFLD24 | DFLD25 | DFLD27 | DFLD30 | DFLD32 | DFLD34 ),
								( DFLD1 | DFLD5 | DFLD6 | DFLD7 | DFLD11 | DFLD12 | DFLD15 | DFLD17 | DFLD22 | DFLD25 | DFLD28 | DFLD33 | DFLD35 ),
								( DFLD1 | DFLD7 | DFLD12 ),
								( DFLD5 | DFLD10 | DFLD14 | DFLD19 | DFLD41 ),
								( DFLD5 | DFLD6 | DFLD10 | DFLD14 | DFLD15 | DFLD16 | DFLD19 | DFLD20 | DFLD24 | DFLD26 | DFLD29 | DFLD33 | DFLD36 | DFLD38 | DFLD41 | DFLD42 ),
								( DFLD1 | DFLD5 | DFLD6 | DFLD7 | DFLD10 | DFLD11 | DFLD14 | DFLD15 | DFLD16 | DFLD17 | DFLD19 | DFLD20 | DFLD21 | DFLD24 | DFLD25 | DFLD26 | DFLD27 | DFLD29 | DFLD30 | DFLD31 | DFLD34 | DFLD37 | DFLD39 | DFLD42 | DFLD43 ),
								( DFLD1 | DFLD5 | DFLD6 | DFLD7 | DFLD10 | DFLD11 | DFLD15 | DFLD16 | DFLD17 | DFLD20 | DFLD21 | DFLD22 | DFLD25 | DFLD27 | DFLD28 | DFLD30 | DFLD32 | DFLD35 | DFLD38 | DFLD40 | DFLD43 | DFLD44 ),
								( DFLD1 | DFLD6 | DFLD7 | DFLD11 | DFLD17 | DFLD22 | DFLD28 | DFLD44 ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL )
} ;

const BITBOARD bbNBBlack[128] = {
// White Turn
								( DFLD7 | DFLD23 | DFLD29 | DFLD34 | DFLD40 | DFLD44 | DFLD45 | DFLD50 ),
								( DFLD7 | DFLD8 | DFLD11 | DFLD13 | DFLD16 | DFLD19 | DFLD21 | DFLD23 | DFLD24 | DFLD26 | DFLD29 | DFLD30 | DFLD31 | DFLD34 | DFLD35 | DFLD36 | DFLD40 | DFLD41 | DFLD44 | DFLD45 | DFLD46 | DFLD50 ),
								( DFLD8 | DFLD9 | DFLD12 | DFLD14 | DFLD17 | DFLD20 | DFLD21 | DFLD22 | DFLD24 | DFLD25 | DFLD26 | DFLD27 | DFLD30 | DFLD31 | DFLD32 | DFLD34 | DFLD35 | DFLD36 | DFLD37 | DFLD40 | DFLD41 | DFLD44 | DFLD45 | DFLD46 | DFLD50 ),
								( DFLD9 | DFLD10 | DFLD13 | DFLD15 | DFLD18 | DFLD22 | DFLD25 | DFLD27 | DFLD31 | DFLD32 | DFLD35 | DFLD36 | DFLD37 | DFLD41 | DFLD45 | DFLD46 ),
								( DFLD10 | DFLD32 | DFLD37 | DFLD41 | DFLD46 ),
								( DFLD39 | DFLD44 | DFLD50 ),
								( DFLD16 | DFLD18 | DFLD23 | DFLD26 | DFLD29 | DFLD34 | DFLD36 | DFLD39 | DFLD40 | DFLD44 | DFLD45 | DFLD46 | DFLD50 ),
								( DFLD17 | DFLD19 | DFLD21 | DFLD24 | DFLD26 | DFLD27 | DFLD29 | DFLD30 | DFLD31 | DFLD34 | DFLD35 | DFLD36 | DFLD37 | DFLD39 | DFLD40 | DFLD41 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD18 | DFLD20 | DFLD22 | DFLD25 | DFLD27 | DFLD28 | DFLD30 | DFLD31 | DFLD32 | DFLD35 | DFLD36 | DFLD37 | DFLD40 | DFLD41 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD28 | DFLD32 | DFLD37 | DFLD41 | DFLD46 | DFLD47 ),
								( NULL ),
								( DFLD33 | DFLD39 | DFLD44 | DFLD50 ),
								( DFLD21 | DFLD23 | DFLD26 | DFLD29 | DFLD31 | DFLD33 | DFLD34 | DFLD36 | DFLD39 | DFLD40 | DFLD41 | DFLD44 | DFLD45 | DFLD46 | DFLD50 ),
								( DFLD22 | DFLD24 | DFLD27 | DFLD30 | DFLD31 | DFLD32 | DFLD34 | DFLD35 | DFLD36 | DFLD37 | DFLD40 | DFLD41 | DFLD42 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD23 | DFLD25 | DFLD28 | DFLD32 | DFLD35 | DFLD37 | DFLD41 | DFLD42 | DFLD45 | DFLD46 | DFLD47 ),
								( DFLD42 | DFLD47 ),
								( DFLD49 ),
								( DFLD26 | DFLD28 | DFLD33 | DFLD36 | DFLD39 | DFLD44 | DFLD46 | DFLD49 | DFLD50 ),
								( DFLD27 | DFLD29 | DFLD31 | DFLD34 | DFLD36 | DFLD37 | DFLD39 | DFLD40 | DFLD41 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD49 | DFLD50 ),
								( DFLD28 | DFLD30 | DFLD32 | DFLD35 | DFLD37 | DFLD38 | DFLD40 | DFLD41 | DFLD42 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD38 | DFLD42 | DFLD47 ),
								( NULL ),
								( DFLD43 | DFLD49 ),
								( DFLD31 | DFLD33 | DFLD36 | DFLD39 | DFLD41 | DFLD43 | DFLD44 | DFLD46 | DFLD49 | DFLD50 ),
								( DFLD32 | DFLD34 | DFLD37 | DFLD40 | DFLD41 | DFLD42 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD33 | DFLD35 | DFLD38 | DFLD42 | DFLD45 | DFLD47 ),
								( NULL ),
								( NULL ),
								( DFLD36 | DFLD38 | DFLD43 | DFLD46 | DFLD49 ),
								( DFLD37 | DFLD39 | DFLD41 | DFLD44 | DFLD46 | DFLD47 | DFLD49 | DFLD50 ),
								( DFLD38 | DFLD40 | DFLD42 | DFLD45 | DFLD47 | DFLD48 | DFLD50 ),
								( DFLD48 ),
								( NULL ),
								( NULL ),
								( DFLD41 | DFLD43 | DFLD46 | DFLD49 ),
								( DFLD42 | DFLD44 | DFLD47 | DFLD50 ),
								( DFLD43 | DFLD45 | DFLD48 ),
								( NULL ),
								( NULL ),
								( DFLD46 | DFLD48 ),
								( DFLD47 | DFLD49 ),
								( DFLD48 | DFLD50 ),
								( DFLD49 ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
// Black Turn
								( DFLD6 | DFLD16 | DFLD18 | DFLD23 | DFLD26 | DFLD29 | DFLD34 | DFLD36 | DFLD39 | DFLD40 | DFLD44 | DFLD45 | DFLD46 | DFLD50 ),
								( DFLD11 | DFLD13 | DFLD16 | DFLD17 | DFLD18 | DFLD19 | DFLD21 | DFLD23 | DFLD24 | DFLD26 | DFLD27 | DFLD29 | DFLD30 | DFLD31 | DFLD34 | DFLD35 | DFLD36 | DFLD37 | DFLD39 | DFLD40 | DFLD41 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD12 | DFLD14 | DFLD17 | DFLD18 | DFLD19 | DFLD20 | DFLD21 | DFLD22 | DFLD24 | DFLD25 | DFLD26 | DFLD27 | DFLD28 | DFLD29 | DFLD30 | DFLD31 | DFLD32 | DFLD34 | DFLD35 | DFLD36 | DFLD37 | DFLD39 | DFLD40 | DFLD41 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD13 | DFLD15 | DFLD18 | DFLD20 | DFLD22 | DFLD25 | DFLD27 | DFLD28 | DFLD30 | DFLD31 | DFLD32 | DFLD35 | DFLD36 | DFLD37 | DFLD40 | DFLD41 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD28 | DFLD32 | DFLD37 | DFLD41 | DFLD46 | DFLD47 ),
								( DFLD33 | DFLD39 | DFLD44 | DFLD50 ),
								( DFLD16 | DFLD18 | DFLD21 | DFLD23 | DFLD26 | DFLD29 | DFLD31 | DFLD33 | DFLD34 | DFLD36 | DFLD39 | DFLD40 | DFLD41 | DFLD44 | DFLD45 | DFLD46 | DFLD50 ),
								( DFLD17 | DFLD19 | DFLD21 | DFLD22 | DFLD23 | DFLD24 | DFLD26 | DFLD27 | DFLD29 | DFLD30 | DFLD31 | DFLD32 | DFLD33 | DFLD34 | DFLD35 | DFLD36 | DFLD37 | DFLD39 | DFLD40 | DFLD41 | DFLD42 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD18 | DFLD20 | DFLD22 | DFLD23 | DFLD24 | DFLD25 | DFLD27 | DFLD28 | DFLD30 | DFLD31 | DFLD32 | DFLD34 | DFLD35 | DFLD36 | DFLD37 | DFLD40 | DFLD41 | DFLD42 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD15 | DFLD23 | DFLD25 | DFLD28 | DFLD32 | DFLD35 | DFLD37 | DFLD41 | DFLD42 | DFLD45 | DFLD46 | DFLD47 ),
								( NULL ),
								( DFLD16 | DFLD26 | DFLD28 | DFLD33 | DFLD36 | DFLD39 | DFLD44 | DFLD46 | DFLD49 | DFLD50 ),
								( DFLD21 | DFLD23 | DFLD26 | DFLD27 | DFLD28 | DFLD29 | DFLD31 | DFLD33 | DFLD34 | DFLD36 | DFLD37 | DFLD39 | DFLD40 | DFLD41 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD49 | DFLD50 ),
								( DFLD22 | DFLD24 | DFLD27 | DFLD28 | DFLD29 | DFLD30 | DFLD31 | DFLD32 | DFLD34 | DFLD35 | DFLD36 | DFLD37 | DFLD38 | DFLD39 | DFLD40 | DFLD41 | DFLD42 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD49 | DFLD50 ),
								( DFLD23 | DFLD25 | DFLD28 | DFLD30 | DFLD32 | DFLD35 | DFLD37 | DFLD38 | DFLD40 | DFLD41 | DFLD42 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD38 | DFLD42 | DFLD47 ),
								( DFLD43 | DFLD49 ),
								( DFLD26 | DFLD28 | DFLD31 | DFLD33 | DFLD36 | DFLD39 | DFLD41 | DFLD43 | DFLD44 | DFLD46 | DFLD49 | DFLD50 ),
								( DFLD27 | DFLD29 | DFLD31 | DFLD32 | DFLD33 | DFLD34 | DFLD36 | DFLD37 | DFLD39 | DFLD40 | DFLD41 | DFLD42 | DFLD43 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD49 | DFLD50 ),
								( DFLD28 | DFLD30 | DFLD32 | DFLD33 | DFLD34 | DFLD35 | DFLD37 | DFLD38 | DFLD40 | DFLD41 | DFLD42 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD50 ),
								( DFLD25 | DFLD33 | DFLD35 | DFLD38 | DFLD42 | DFLD45 | DFLD47 ),
								( NULL ),
								( DFLD26 | DFLD36 | DFLD38 | DFLD43 | DFLD46 | DFLD49 ),
								( DFLD31 | DFLD33 | DFLD36 | DFLD37 | DFLD38 | DFLD39 | DFLD41 | DFLD43 | DFLD44 | DFLD46 | DFLD47 | DFLD49 | DFLD50 ),
								( DFLD32 | DFLD34 | DFLD37 | DFLD38 | DFLD39 | DFLD40 | DFLD41 | DFLD42 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD48 | DFLD49 | DFLD50 ),
								( DFLD33 | DFLD35 | DFLD38 | DFLD40 | DFLD42 | DFLD45 | DFLD47 | DFLD48 | DFLD50 ),
								( DFLD48 ),
								( NULL ),
								( DFLD36 | DFLD38 | DFLD41 | DFLD43 | DFLD46 | DFLD49 ),
								( DFLD37 | DFLD39 | DFLD41 | DFLD42 | DFLD43 | DFLD44 | DFLD46 | DFLD47 | DFLD49 | DFLD50 ),
								( DFLD38 | DFLD40 | DFLD42 | DFLD43 | DFLD44 | DFLD45 | DFLD47 | DFLD48 | DFLD50 ),
								( DFLD35 | DFLD43 | DFLD45 | DFLD48 ),
								( NULL ),
								( DFLD36 | DFLD46 | DFLD48 ),
								( DFLD41 | DFLD43 | DFLD46 | DFLD47 | DFLD48 | DFLD49 ),
								( DFLD42 | DFLD44 | DFLD47 | DFLD48 | DFLD49 | DFLD50 ),
								( DFLD43 | DFLD45 | DFLD48 | DFLD49 | DFLD50 ),
								( DFLD49 ),
								( NULL ),
								( DFLD46 | DFLD48 ),
								( DFLD47 | DFLD49 ),
								( DFLD48 | DFLD50 ),
								( DFLD45 | DFLD49 ),
								( NULL ),
								( DFLD46 | DFLD47 ),
								( DFLD47 | DFLD48 ),
								( DFLD48 | DFLD49 ),
								( DFLD49 | DFLD50 ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL ),
								( NULL )
} ;

#define DFLD_POSITION_6_15						( DFLD6 | DFLD7 | DFLD8 | DFLD9 | DFLD10 | DFLD11 | DFLD12 | DFLD13 | DFLD14 | DFLD15 )
#define DFLD_POSITION_36_45						( DFLD36 | DFLD37 | DFLD38 | DFLD39 | DFLD40 | DFLD41 | DFLD42 | DFLD43 | DFLD44 | DFLD45 )

#define DFLD_ROW0								( DFLD1 | DFLD2 | DFLD3 | DFLD4 | DFLD5 )
#define DFLD_ROW1								( DFLD6 | DFLD7 | DFLD8 | DFLD9 | DFLD10 )
#define DFLD_ROW2								( DFLD11 | DFLD12 | DFLD13 | DFLD14 | DFLD15 )
#define DFLD_ROW3								( DFLD16 | DFLD17 | DFLD18 | DFLD19 | DFLD20 )
#define DFLD_ROW4								( DFLD21 | DFLD22 | DFLD23 | DFLD24 | DFLD25 )
#define DFLD_ROW5								( DFLD26 | DFLD27 | DFLD28 | DFLD29 | DFLD30 )
#define DFLD_ROW6								( DFLD31 | DFLD32 | DFLD33 | DFLD34 | DFLD35 )
#define DFLD_ROW7								( DFLD36 | DFLD37 | DFLD38 | DFLD39 | DFLD40 )
#define DFLD_ROW8								( DFLD41 | DFLD42 | DFLD43 | DFLD44 | DFLD45 )
#define DFLD_ROW9								( DFLD46 | DFLD47 | DFLD48 | DFLD49 | DFLD50 )

#define DFLD_COLUMN0							( DFLD6 | DFLD16 | DFLD26 | DFLD36 | DFLD46 )
#define DFLD_COLUMN1							( DFLD1 | DFLD11 | DFLD21 | DFLD31 | DFLD41 )
#define DFLD_COLUMN2							( DFLD7 | DFLD17 | DFLD27 | DFLD37 | DFLD47 )
#define DFLD_COLUMN3							( DFLD2 | DFLD12 | DFLD22 | DFLD32 | DFLD42 )
#define DFLD_COLUMN4							( DFLD8 | DFLD18 | DFLD28 | DFLD38 | DFLD48 )
#define DFLD_COLUMN5							( DFLD3 | DFLD13 | DFLD23 | DFLD33 | DFLD43 )
#define DFLD_COLUMN6							( DFLD9 | DFLD19 | DFLD29 | DFLD39 | DFLD49 )
#define DFLD_COLUMN7							( DFLD4 | DFLD14 | DFLD24 | DFLD34 | DFLD44 )
#define DFLD_COLUMN8							( DFLD10 | DFLD20 | DFLD30 | DFLD40 | DFLD50 )
#define DFLD_COLUMN9							( DFLD5 | DFLD15 | DFLD25 | DFLD35 | DFLD45 )

#define DFLD_PROMOTION_WHITE					( DFLD1 | DFLD2 | DFLD3 | DFLD4 | DFLD5 )
#define DFLD_PROMOTION_BLACK					( DFLD46 | DFLD47 | DFLD48 | DFLD49 | DFLD50 )

#define DFLD_BREAKTHROUGH_WHITE					( DFLD_ROW1 | DFLD_ROW2 | DFLD_ROW3 | DFLD_ROW4 | DFLD_ROW5 )
#define DFLD_BREAKTHROUGH_BLACK					( DFLD_ROW4 | DFLD_ROW5 | DFLD_ROW6 | DFLD_ROW7 | DFLD_ROW8 )

#define DFLD_ALL								( DFLD_ROW0 | DFLD_ROW1 | DFLD_ROW2 | DFLD_ROW3 | DFLD_ROW4 | DFLD_ROW5 | DFLD_ROW6 | DFLD_ROW7 | DFLD_ROW8 | DFLD_ROW9 )

#define DFLD_BLACKFROMTO_6						( DFLD1 )
#define DFLD_BLACKFROMTO_10						( DFLD4 | DFLD5 )
#define DFLD_BLACKFROMTO_11						( DFLD1 | DFLD2 | DFLD6 | DFLD7 )
#define DFLD_BLACKFROMTO_15						( DFLD4 | DFLD5 | DFLD10 )
#define DFLD_BLACKFROMTO_20						( DFLD3 | DFLD4 | DFLD5 | DFLD9 | DFLD10 | DFLD14 | DFLD15 )
#define DFLD_BLACKFROMTO_21						( DFLD1 | DFLD2 | DFLD3 | DFLD6 | DFLD7 | DFLD8 | DFLD11 | DFLD12 | DFLD16 | DFLD17 )
#define DFLD_BLACKFROMTO_25						( DFLD3 | DFLD4 | DFLD5 | DFLD9 | DFLD10 | DFLD14 | DFLD15 | DFLD20 )
#define DFLD_BLACKFROMTO_16						( DFLD1 | DFLD2 | DFLD6 | DFLD7 | DFLD11 )
#define DFLD_BLACKFROMTO_26						( DFLD1 | DFLD2 | DFLD3 | DFLD6 | DFLD7 | DFLD8 | DFLD11 | DFLD12 | DFLD16 | DFLD17 | DFLD21 )

#define DFLD_WHITEFROMTO_45						( DFLD50 )
#define DFLD_WHITEFROMTO_41						( DFLD46 | DFLD47 )
#define DFLD_WHITEFROMTO_40						( DFLD44 | DFLD45 | DFLD49 | DFLD50 )
#define DFLD_WHITEFROMTO_35						( DFLD40 | DFLD44 | DFLD45 | DFLD49 | DFLD50 )
#define DFLD_WHITEFROMTO_30						( DFLD34 | DFLD35 | DFLD39 | DFLD40 | DFLD43 | DFLD44 | DFLD45 | DFLD48 | DFLD49 | DFLD50 )
#define DFLD_WHITEFROMTO_36						( DFLD41 | DFLD46 | DFLD47 )
#define DFLD_WHITEFROMTO_31						( DFLD36 | DFLD37 | DFLD41 | DFLD42 | DFLD46 | DFLD47 | DFLD48 )
#define DFLD_WHITEFROMTO_26						( DFLD31 | DFLD36 | DFLD37 | DFLD41 | DFLD42 | DFLD46 | DFLD47 | DFLD48 )
#define DFLD_WHITEFROMTO_25						( DFLD30 | DFLD34 | DFLD35 | DFLD39 | DFLD40 | DFLD43 | DFLD44 | DFLD45 | DFLD48 | DFLD49 | DFLD50 )

const BITBOARD bbWhitePromotion[51] =	{	NULL, NULL, NULL, NULL, NULL, NULL,	// No White Promotion for Position 0 - 5
											DFLD1,								// Position 6
											DFLD1 | DFLD2,						// Position 7
											DFLD2 | DFLD3,						// Position 8
											DFLD3 | DFLD4,						// Position 9
											DFLD4 | DFLD5,						// Position 10
											DFLD1 | DFLD2,						// Position 11
											DFLD1 | DFLD2 | DFLD3,				// Position 12
											DFLD2 | DFLD3 | DFLD4,				// Position 13
											DFLD3 | DFLD4 | DFLD5,				// Position 14
											DFLD4 | DFLD5						// Position 15
} ;



// MoveGenerator Move codes
#define WHITEMANMOVE		1
#define WHITEMANCAPTURE		2
#define WHITEKINGMOVE		3
#define WHITEKINGCAPTURE	4
#define BLACKMANMOVE		5
#define BLACKMANCAPTURE		6
#define BLACKKINGMOVE		7
#define BLACKKINGCAPTURE	8

#define MOVE_KILL		0x1
#define MOVE_KING		0x2
#define MOVE_BLACK		0x4

// Move PACKED values
#define LOOK_MOVE_PACKED_CODE	6
#define LOOK_MOVE_PACKED_FROMTO	6
#define LOOK_MOVE_BITS_CODE		0x3F
#define LOOK_MOVE_BITS_FROMTO	0x3F

// Score Values
#define WIN_THRESHOLD		4000
#define WIN_VAL				5000
#define DRAW_VAL			0
#define LOOSE_THRESHOLD		-4000
#define LOOSE_VAL			-5000

#define INFINITE_VAL		1000000		// Arbitrary large value

#define DRAW_REPETITION		2		// In theory it is 3, but 2 is more efficient for tree-search
#define DRAW_REPETITION_PLY	3

#define DATABASE_WIN_VAL	4500
#define DATABASE_DRAW_VAL	0
#define DATABASE_LOOSE_VAL	-4500

// DXP
#define DXP_WIN					0
#define DXP_DRAW				1
#define DXP_LOSE				2
#define DXP_UNKNOWN				3

#define DXP_DEFAULT_MAXGAMES	158
#define DXP_DEFAULT_MAXMOVES	65
#define DXP_DEFAULT_MAXTIME		10

// Eval Exact Flags
#define EVAL_EXACT_LOOSE	0x01
#define EVAL_EXACT_LEDRAW	0x02
#define EVAL_EXACT_DRAW		0x04
#define EVAL_EXACT_GEDRAW	0x08
#define EVAL_EXACT_WIN		0x10
#define EVAL_EXACT_ALLFLAGS	0x1F
#define EVAL_EXACT_WDLFLAGS	0x11

#define EVAL_EXACT_COUNT6	6
#define EVAL_EXACT_COUNT7	7
#define EVAL_EXACT_COUNT8	8
#define EVAL_EXACT_COUNT9	9
#define EVAL_EXACT_COUNT10	10

#define	OUTPUT_MAX_MESSAGE_LENGTH	256
#define ENGINE_OUTPUT_STACK_SIZE	1024

// 64Bit Intrinsics
#define POPCNT64	__popcnt64
#define BITSCANFORWARD64 _BitScanForward64
#define BITSCANREVERSE64 _BitScanReverse64

/*
inline int POPCNT64(BITBOARD bbBitBoard)\
{ int iCount=0 ; while ( bbBitBoard ) { bbBitBoard &= ( bbBitBoard - 1 ) ; iCount++ ; } ; return ( iCount ) ; }

#define BITSCANFORWARD64(iBit, bbBitBoard)\
{ unsigned int iBitBoard ;\
	if ( bbBitBoard & 0xFFFFFFFF ) { iBitBoard =  (int)(bbBitBoard & 0xFFFFFFFF) ; _BitScanForward(iBit, iBitBoard) ;\
	} else { iBitBoard = (int)(bbBitBoard>>32) ; _BitScanForward(iBit, iBitBoard) ; *iBit += 32 ; } }

#define BITSCANREVERSE64(iBit, bbBitBoard)\
{ unsigned int iBitBoard ;\
	if ( bbBitBoard>>32 ) { iBitBoard =  (int)(bbBitBoard>>32) ; _BitScanReverse(iBit, iBitBoard) ; *iBit += 32 ;\
	} else { iBitBoard = (int)(bbBitBoard & 0xFFFFFFFF) ; _BitScanReverse(iBit, iBitBoard) ; } }
*/

#define BB_WHITEMAN		0
#define BB_WHITEKING	1
#define BB_BLACKMAN		2
#define BB_BLACKKING	3
#define BB_EMPTY		4
#define BB_OPPONENT		5
#define BB_WHITEPIECE	6
#define BB_BLACKPIECE	7

#define NROOT		1024			// Root size

// EndGameDatabaseGen default parameters
#define ENDGAMEDBGEN_PIECE_DEFAULT	4

// Script Structure
#define SCRIPT_MAX_COMMAND	1024

// Log parameters
#define LOG_PLY_DEFAULT		15
#define LOG_FILE			"Damage.log"
#define LOG_NGAME			0x1
#define LOG_POSITION		0x2
#define LOG_NMOVE			0x4
#define LOG_SEARCH			0x8
#define LOG_DEFAULT_MODE	( LOG_NGAME | LOG_POSITION  | LOG_NMOVE | LOG_SEARCH )

// HashTable default parameters
#define HASHTABLESIZE_DEFAULT			23
#define HASHTABLESIZE_DEFAULT_NOTABLE	19

// SingularNode Detection/Extension
#define SINGULAREXTENSIONDEPTH_DEFAULT	8

// DB EndGame default parameters
#define DBMAXPIECE_DEFAULT	7
#define DBCACHESIZE_DEFAULT	20
#define DBCACHESIZE_DEFAULT_NOCACHE	15
#define DBCACHESIZE_DEFAULT_DAMAGE	20
#define DBCACHESIZE_DEFAULT_HORIZON	17

// SSearch
#define SSEARCH_SEARCHDEPTH			5
#define SSEARCH_SEARCHSTARTDEPTH	10
#define SSEARCH_NOMOVE				-1
#define SSEARCH_MARGIN				50

typedef struct {
	CString csWhite ;
	CString csBlack ;
	CString csTournament ;
	CString csSite ;
	
	int iCRound ;
	int iCDate ;

	int iRound ;
	int iYear ;
	int iMonth ;
	int iDay ;
	int iResult ;
} GAMEHEADER, *PGAMEHEADER ;

typedef struct {
	bool		bScript ;
	int			iCommand ;
	int			iNCommand ;
	CString		csScriptFileName ;
	CString		csCommand[SCRIPT_MAX_COMMAND] ;
} SCRIPT, *PSCRIPT ;

typedef struct {
	bool bLogOutput ;
	bool bOutput ;
	int iOutputMode ;
	int iNPiece ;
	int iNWhiteMan ;
	int iNWhiteKing ;
	int iNBlackMan ;
	int iNBlackKing ;
	int iWhiteSlice ;
	int iBlackSlice ;
} ENDGAMEDBGEN, *PENDGAMEDBGEN ;

typedef struct {
	__int64 iDBID ;
	__int64 iCount ;
	__int64 iCountExact ;
	__int64 iNWin ;
	__int64 iNDraw ;
	__int64 iNLose ;
	__int64 iNBlocks ;
	__int64 iSizeDB ;
} DB64HEADER, *PDB64HEADER ;

typedef struct {

	unsigned char*	pDBCache ;
	unsigned char*	pDBCache0 ;

	int				iNFiles ;
	int				iNFilesOpen ;

	int				iDBCacheSize ;

	DBINDEX			iNCacheRead ;
	DBINDEX			iNCacheHeuristic ;
	DBINDEX			iNCacheHeuristicError ;
	DBINDEX			iNCacheWrite ;

	DBINDEX			iDBCount ;
	DBINDEX			iDBIndexCount ;	
	DBINDEX			iDBIndexTable ;

	DBINDEX			iCacheLRU ;
	DBINDEX			iCacheMRU ;
	DBINDEX			iCacheFreeBlocks ;

	DBINDEX*		pDBIndexTable ;
	DBINDEX*		pDBCacheIndexTable ;		
	int*			pDBCacheIDTable ;

	DBINDEX*		pDBCachePrevBlock ;
	DBINDEX*		pDBCacheNextBlock ;

	DBINDEX			iDB64BlockStart[4096 * 81] ;
	HANDLE			hFileDB64[4096 * 81] ;
	DB64HEADER		DB64Header[4096 * 81] ;
	
} DB64CACHE, *PDB64CACHE ;

typedef struct {
	bool			bExact ;
	unsigned char*	pcDB ;
	__int64*		pDBIndex ;
	int				iCurrentByte ;
	int				iCurrentByteX ;
	__int64			iCurrentIndex ;
	__int64			iCompressedByteX ;
	int				iLastByte ;
	int				iLastCount ;
	__int64			iDBIndex ;
} COMPRESSDB, *PCOMPRESSDB ;

typedef struct {
	int	iMoveCode ;
	BITBOARD bbFrom ;
	BITBOARD bbTo ;
	BITBOARD bbCapture ;
	int index ;
	unsigned int flags ;
} LOOK, *PLOOK ;

// Search
#define HASH_LOW			0x1
#define HASH_HIGH			0x2
#define HASH_BOUND			0x3
#define HASH_SNGMOVE		0x4
#define HASH_WHITEMOVE		0x8
#define HASH_BLACKMOVE		0x10
#define HASH_KILLMOVE		0x20
#define HASH_TIMESTAMP		0x40

#define HASH_EXACT_LOOSE	0x80
#define HASH_EXACT_LEDRAW	0x100
#define HASH_EXACT_DRAW		0x200
#define HASH_EXACT_GEDRAW	0x400
#define HASH_EXACT_WIN		0x800
#define HASH_EXACT_ALLFLAGS	0xF80
#define HASH_EXACT_WDLFLAGS	0x880

#define HASH_EVALFLAGS		7	

#define HASH_READ_ERROR			0
#define HASH_READ_MOVE			1
#define HASH_READ_NOMOVE		-1
#define HASH_READ_NODEPTH		-1
#define HASH_READ_SCORE			2
#define HASH_READ_EXCLUDEMOVE	4
#define HASH_WRITE_EXCLUDEMOVE	4
#define HASH_READ_EVALUATION	6
#define HASH_WRITE_EVALUATION	6
#define HASH_READ_DB64			7
#define HASH_WRITE_DB64			7

#define HASH_CLUSTER_SIZE		4

// Hash Packed values
#define HASH_PACKED_SCORE		16			// Nr of bits packed
#define HASH_PACKED_FLAGS		12
#define HASH_PACKED_INDEX		10
#define HASH_PACKED_DEPTH		8
#define HASH_PACKED_MOVE		18

#define HASH_PACKED_BITPOSITION_SCORE	0
#define HASH_PACKED_BITPOSITION_FLAGS	16
#define HASH_PACKED_BITPOSITION_INDEX	28
#define HASH_PACKED_BITPOSITION_DEPTH	38
#define HASH_PACKED_BITPOSITION_MOVE	46	

#define HASH_PACKED_FLAGS_BIT_WHITEMOVE	0x80000
#define HASH_PACKED_FLAGS_BIT_BLACKMOVE	0x100000
#define HASH_PACKED_FLAGS_BIT_TIMESTAMP	0x400000

#define HASH_BITMASK_SCORE		0xFFFF
#define HASH_BITMASK_FLAGS		0xFFF
#define HASH_BITMASK_INDEX		0x3FF
#define HASH_BITMASK_DEPTH		0xFF
#define HASH_BITMASK_MOVE		0x3FFFF

#define SINGULAR_VALUE	25	
#define SINGULAR_FLAG	0x100000		 

// Parallel Processing & Search-Thread
#define PP_MODE_DEFAULT							0
#define PP_MODE_MAXMODE							3

#define PP_MODE_SINGLECORE						0
#define PP_MODE_MM_SINGLECORE					1
#define PP_MODE_MULTICORE_COPYSEARCH			2
#define PP_MODE_MULTICORE_SMP_SEARCH			3

#define PP_SUBSEARCHMODE_DEFAULT				0
#define PP_SUBSEARCHMODE_PVSPLIT_NORMAL			0
#define PP_SUBSEARCHMODE_PVSPLIT_ROOT			1
#define PP_SUBSEARCHMODE_PVSPLIT_MODIFIED		2

#define PP_PROCESS_NOTAVAILABLE					0
#define PP_PROCESS_DEACTIVATED					1
#define PP_PROCESS_IDLE							2
#define PP_PROCESS_RUN							3

#define PP_SEARCHTHREAD_IDLE					1
#define PP_SEARCHTHREAD_READY					2
#define PP_SEARCHTHREAD_RUN						4
#define PP_SEARCHTHREAD_SINGLE					8
#define PP_SEARCHTHREAD_ALL						16

#define PP_PVSPLIT_SPLITDEPTH_THRESHOLD			5

#define PP_MAXTHREADSPERSPLITPOINT				4
#define PP_MAXACTIVESPLITPOINTS					8
#define PP_MINIMUMSPLITBRANCH					0
#define PP_MINIMUMSPLITDEPTH					5

#define PP_MODE_MAX						2
#define SEARCHTHREAD_MAXTHREAD			16
#define SEARCHTHREAD_PARALLEL			true

#define SEARCHTHREAD_SHARED_HASHTABLE		true
#define SEARCHTHREAD_SHARED_EVALHASHTABLE	false
#define SEARCHTHREAD_SHARED_HISTORY			false
#define SEARCHTHREAD_PACKED_HASHTABLE		true

#define THREAD_TASK_SEARCH					0
#define THREAD_TASK_MM_SEARCH				1
#define THREAD_TASK_SMP_SEARCH				2

// Evaluation
#define MAN_VAL			100
#define KING_VAL		320
#define WSTONE_VAL		100
#define WMAN_VAL		100
#define WKING_VAL		320
#define BSTONE_VAL		100
#define BMAN_VAL		100
#define BKING_VAL		320
#define ISEVAL			150

// Bitboard Patterns
#define BITBOARD_PATTERN_P7_P14					( DFLD7  | DFLD8  | DFLD9  | DFLD10 | DFLD11 | DFLD12 | DFLD13 | DFLD14 )
#define BITBOARD_PATTERN_P37_P44				( DFLD37 | DFLD38 | DFLD39 | DFLD40 | DFLD41 | DFLD42 | DFLD43 | DFLD44 )

#define BITBOARD_PATTERN_PB3_PB26				( DFLD3 | DFLD8 | DFLD12 | DFLD17 | DFLD21 | DFLD26 )
#define BITBOARD_PATTERN_PB4_PB36				( DFLD4 | DFLD9 | DFLD13 | DFLD18 | DFLD22 | DFLD27 | DFLD31 | DFLD36 )
#define BITBOARD_PATTERN_PB1_PB45				( DFLD1 | DFLD7 | DFLD12 | DFLD18 | DFLD23 | DFLD29 | DFLD34 | DFLD40 | DFLD45 )
#define BITBOARD_PATTERN_PB2_PB35				( DFLD2 | DFLD8 | DFLD13 | DFLD19 | DFLD24 | DFLD30 | DFLD35 )

#define BITBOARD_PATTERN_PB15_PB47				( DFLD15 | DFLD20 | DFLD24 | DFLD29 | DFLD33 | DFLD38 | DFLD42 | DFLD47 ) 
#define BITBOARD_PATTERN_PB25_PB48				( DFLD25 | DFLD30 | DFLD34 | DFLD39 | DFLD43 | DFLD48 )
#define BITBOARD_PATTERN_PB6_PB50				( DFLD6  | DFLD11 | DFLD17 | DFLD22 | DFLD28 | DFLD33 | DFLD39 | DFLD44 | DFLD50 ) 
#define BITBOARD_PATTERN_PB16_PB49				( DFLD16 | DFLD21 | DFLD27 | DFLD32 | DFLD38 | DFLD43 | DFLD49 )
#define BITBOARD_PATTERN_PB5_PB46				( DFLD5  | DFLD10 | DFLD14 | DFLD19 | DFLD23 | DFLD28 | DFLD32 | DFLD37 | DFLD41 | DFLD46 )

#define BITBOARD_PATTERN_AREA_PB1_PB3_PB26		( DFLD1 | DFLD2 | DFLD3  | DFLD6  | DFLD7  | DFLD8  | DFLD11 | DFLD12 | DFLD16 | DFLD17 | DFLD21 | DFLD26 )
#define BITBOARD_PATTERN_AREA_PB1_PB2_PB16		( DFLD1 | DFLD2 | DFLD6  | DFLD7  | DFLD11 | DFLD16 )
#define BITBOARD_PATTERN_AREA_PB2_PB5_PB35		( DFLD2 | DFLD3 | DFLD4  | DFLD5  | DFLD8  | DFLD9  | DFLD10 | DFLD13 | DFLD14 | DFLD15 | DFLD19 | DFLD20 | DFLD24 | DFLD25 | DFLD30 | DFLD35 ) 
#define BITBOARD_PATTERN_AREA_PB3_PB5_PB25		( DFLD3 | DFLD4 | DFLD5  | DFLD9  | DFLD10 | DFLD14 | DFLD15 | DFLD20 | DFLD25 )
#define BITBOARD_PATTERN_AREA_PB1_PB4_PB36		( DFLD1 | DFLD2 | DFLD3  | DFLD4  | DFLD6  | DFLD7  | DFLD8  | DFLD9 | DFLD11 | DFLD12 | DFLD13 | DFLD16 | DFLD17 | DFLD18 | \
													     DFLD21 | DFLD22 | DFLD26 | DFLD27 | DFLD31 | DFLD36 )

#define BITBOARD_PATTERN_AREA_PB25_PB48_PB50	( DFLD25 | DFLD30 | DFLD34 | DFLD35 | DFLD39 | DFLD40 | DFLD43 | DFLD44 | DFLD45 | DFLD48 | DFLD49 | DFLD50 )
#define BITBOARD_PATTERN_AREA_PB35_PB49_PB50	( DFLD35 | DFLD40 | DFLD44 | DFLD45 | DFLD49 | DFLD50 )
#define BITBOARD_PATTERN_AREA_PB16_PB46_PB49	( DFLD16 | DFLD21 | DFLD26 | DFLD27 | DFLD31 | DFLD32 | DFLD36 | DFLD37 | DFLD38 | \
														   DFLD41 | DFLD42 | DFLD43 | DFLD46 | DFLD47 | DFLD48 | DFLD49 )
#define BITBOARD_PATTERN_AREA_PB26_PB46_PB48	( DFLD26 | DFLD31 | DFLD36 | DFLD37 | DFLD41 | DFLD42 | DFLD46 | DFLD47 | DFLD48 )
#define BITBOARD_PATTERN_AREA_PB15_PB47_PB50	( DFLD15 | DFLD20 | DFLD24 | DFLD25 | DFLD29 | DFLD30 | DFLD33 | DFLD34 | DFLD35 | DFLD38 | DFLD39 | DFLD40 | \
														   DFLD42 | DFLD43 | DFLD44 | DFLD45 | DFLD47 | DFLD48 | DFLD49 |DFLD50 )

#define BITBOARD_PATTERN_WHITE_AREA1_FRONT		( DFLD1  | DFLD2  | DFLD6  | DFLD7  | DFLD11 | DFLD12 | DFLD16 | DFLD17 | DFLD21 | DFLD22 | DFLD26 | DFLD27 )
#define BITBOARD_PATTERN_WHITE_AREA1_BASE		( DFLD31 | DFLD32 | DFLD36 | DFLD37 | DFLD41 | DFLD42 | DFLD46 | DFLD47 )
#define BITBOARD_PATTERN_WHITE_AREA2			( DFLD3  | DFLD8  | DFLD13 | DFLD18 | DFLD23 | DFLD28 | DFLD33 | DFLD38 | DFLD43 | DFLD48 )
#define BITBOARD_PATTERN_WHITE_AREA3_FRONT		( DFLD4  | DFLD5  | DFLD9  | DFLD10 | DFLD14 | DFLD15 | DFLD19 | DFLD20 | DFLD24 | DFLD25 | DFLD29 | DFLD30 )
#define BITBOARD_PATTERN_WHITE_AREA3_BASE		( DFLD34 | DFLD35 | DFLD39 | DFLD40 | DFLD44 | DFLD45 | DFLD49 | DFLD50 )

#define BITBOARD_PATTERN_BLACK_AREA1_BASE		( DFLD1  | DFLD2  | DFLD6  | DFLD7  | DFLD11 | DFLD12 | DFLD16 | DFLD17 )
#define BITBOARD_PATTERN_BLACK_AREA1_FRONT		( DFLD21 | DFLD22 | DFLD26 | DFLD27 | DFLD31 | DFLD32 | DFLD36 | DFLD37 | DFLD41 | DFLD42 | DFLD46 | DFLD47 )
#define BITBOARD_PATTERN_BLACK_AREA2			( DFLD3  | DFLD8  | DFLD13 | DFLD18 | DFLD23 | DFLD28 | DFLD33 | DFLD38 | DFLD43 | DFLD48 )
#define BITBOARD_PATTERN_BLACK_AREA3_BASE		( DFLD4  | DFLD5  | DFLD9  | DFLD10 | DFLD14 | DFLD15 | DFLD19 | DFLD20 )
#define BITBOARD_PATTERN_BLACK_AREA3_FRONT		( DFLD24 | DFLD25 | DFLD29 | DFLD30 | DFLD34 | DFLD35 | DFLD39 | DFLD40 | DFLD44 | DFLD45 | DFLD49 | DFLD50 )

#define	INIT_POSITION							(	DFLD1  | DFLD2  | DFLD3  | DFLD4 |  DFLD5  | DFLD6  | DFLD7  | DFLD8  | DFLD9  | DFLD10 | \
													DFLD11 | DFLD12 | DFLD13 | DFLD14 | DFLD15 | DFLD16 | DFLD17 | DFLD18 | DFLD19 | DFLD20 | \
													DFLD21 | DFLD22 | DFLD23 | DFLD24 | DFLD25 | DFLD26 | DFLD27 | DFLD28 | DFLD29 | DFLD30 | \
													DFLD31 | DFLD32 | DFLD33 | DFLD34 | DFLD35 | DFLD36 | DFLD37 | DFLD38 | DFLD39 | DFLD40 | \
													DFLD41 | DFLD42 | DFLD43 | DFLD44 | DFLD45 | DFLD46 | DFLD47 | DFLD48 | DFLD49 | DFLD50 )

// Some inline Functions
#define FINDLSB(x)		( x & ( ~x + 1 ) )
#define QBITSET(x,y)	( x>>y & 1 )
#define QBITCNT1(x)		( ( x & (x-1) ) == 0 )
#define QBITCNTN1(x)	( ( x & (x-1) ) != 0 )
#define BITSET(x,y)		( x |= ( (BITBOARD)1<<y) )

#define QAll(bitboard, bits)		( ( (bitboard) & ( bits ) ) == ( bits ) )
#define QAllBlackMan(bits)			QAll( bbBlackMan, bits )
#define QAllWhiteMan(bits)			QAll( bbWhiteMan, bits )
#define QallWhiteManEmpty(bits)		QAll( bbWhiteMan | bbEmpty, bits )
#define QAllBlackManEmpty(bits)		QAll( bbBlackMan | bbEmpty, bits )

#define ALLPIECE(x,y)	( ( x & y ) == y )

#define TBKING  -2
#define TBMAN   -1
#define TEMPTY   0
#define TWKING   2
#define TWMAN    1
#define TINVALID 3

#define TWTURN		1
#define TBTURN		0

#define TTOTAAL     10
#define TMATERIAAL  11
#define TDOORBRAKEN 12
#define TFORMATIONS 13
#define THANGEND    14
#define TTEMPOS     15
#define TVLEUGELS   16
#define TVERDELING  17
#define THARMONIEUS 18
#define TVOORPOSTEN 19
#define TPATRONEN   20

#define COUNT_BLOCK			70000L
#define BFND_BLOCK			70000L

// PV Definitions
#define MAX_PV					32 
#define PV_MASK					0xFFFFF
#define PV_MASK_FLAG			0x300000
#define PV_FLAG_SINGULAR		0x100000
#define PV_FLAG_SSEARCH			0x200000

#define PVPOSITION_INIT			0
#define PVPOSITION_COPY			1

#define PVPOSITION_INIT_NONE		0
#define PVPOSITION_INIT_EVAL		1
#define PVPOSITION_INIT_DATABASE	2
#define PVPOSITION_INIT_HASH		3
#define PVPOSITION_INIT_EVALHASH	4

// Match Definitions

// Array max-size
#define GAMEHISTORY_MAXSIZE	513

// HistoryTable Size
#define HISTORYTABLE_SIZE	4096

// MoveStack Size
#define MOVESTACK_SIZE		4096

// TreeStack Size
#define TREESTACK_SIZE		128

// Table size (Eval & Learn )
#define EVAL_HASHTABLE				131072		// Eval
#define EVAL_HASHTABLE_MASK			0x1FFFF
#define LEARN_HASHTABLE				1048576		// Learn
#define LEARN_HASHTABLE_MASK		0xFFFFF

// KR Define

// Definition of a board. bit 0 means square 1, bit 1 is square 2, ...
typedef struct {
	unsigned __int64 bbBlack ;
	unsigned __int64 bbWhite ;
	unsigned __int64 bbKing ;
} KRBOARD, *PKRBOARD ;

typedef struct {
	int					iWhiteSlice ;
	int					iBlackSlice ;
	unsigned __int64	iDBIndex ;
	KRBOARD				KRBoard ;
	BITBOARD			bbBitBoard[5] ;
} KRPOS, *PKRPOS ;

// Structure definitions
typedef struct {
	bool				bTurn ;
	HASHKEY				HashKey ;
	BITBOARD			bbField[4] ;
} DEBUGDATA, *PDEBUGDATA ;

typedef struct {
	CString				csLogMessage ;
} LOGDATA, *PLOGDATA ;

typedef struct {
	bool				bAdjudicate ;
	bool				bAlternate ;
	bool				bPDN ;
	bool				bFileOpen ;
	bool				bMatchLogFileOpen ;
	bool				bDXPClientTurn ;
	bool				bDXPGameEnd ;
	int					iMatchMode ;
	int					iMaxTime ;
	int					iMaxMoves ;
	int					iWhiteMoves ;
	int					iBlackMoves ;
	int					iMaxGames ;
	int					iNGames ;
	int					iNWin ;
	int					iNDraw ;
	int					iNLoose ;
	int					iNUnknown ;
	int					iLog ;
	int					iGameResult ;
	int					iResult[158] ;
	int					iClientResult[158] ;
	int					iParamP1[32] ;
	int					iParamP2[32] ;
	BITBOARD			bbField[5] ;
	CFile				MatchFile ;
	FILE*				pMatchLogFile ;
	struct timeb		DXPGameStart ;
	struct timeb		DXPGameEnd ;
	CString				csBallotWhiteMove ;
	CString				csBallotBlackMove ;
	CString				csMatchName ;
	CString				csMatchFile ;
	CString				csMatchLogFile ;
	CString				csClientName ;
	CString				csServerName ;
} DXPMATCH, *PDXPMATCH ;

typedef struct {
	bool				bActive ;
	int					index ;
	int					score ;
} ROOT , *PROOT ;

typedef struct {
	int					iIndex ;
	int					iScore ;
	int					iSearchTime ;
} ROOTDEPTHINFO, *PROOTDEPTHINFO ;

typedef struct {
	HASHCODE			hcode ;	
	unsigned int		hddr ;
	int					hval ;
	unsigned short int	bm_idx ;
	unsigned int		flags ;
	unsigned int		hmove ;
} HAREC, *PHAREC ;

typedef struct {
	HASHKEY				HashKey[HASH_CLUSTER_SIZE] ;	
	HASHDATA			HashData[HASH_CLUSTER_SIZE] ;
} HARECPP, *PHARECPP ;

typedef struct {
	HASHCODE			hcode ;
	unsigned int		flags ;
	int					hval ;
} EHAREC, *PEHAREC ;		

typedef struct {
	HASHKEY				HashKey ;	
	HASHDATA			HashData ;
} DHAREC, *PDHAREC ;

typedef struct {
	bool			bTurn ;
	int				iType ;
	int				iDepth ;
	int				iScore ;
	BITBOARD		bbField[8] ;
} PVPOSITION, *PPVPOSITION ;

typedef struct {
	BITBOARD bbField[8] ;
} BBPOSITION, *PBBPOSITION ;

typedef struct {
	bool bTurn ;
	int iNWhiteMan ;
	int iNWhiteKing ;
	int iNBlackMan ;
	int iNBlackKing ;
	BITBOARD bbField[4] ;
} DB64POSITION, *PDB64POSITION ;

typedef struct {
	bool bFinished ;
	bool bPV ;

	int iMaster, iSlaves[SEARCHTHREAD_MAXTHREAD] ;
	int iDepth ;
	int	iPly ;
	int iMoves ;
	int iNMoves ;
	int iSplitPointID ;
	unsigned int	imx_PV ;
	unsigned int	PV[MAX_PV] ;
	PLOOK pLook ;
	volatile int iAlfa, iBeta, iBMIndex, iScore, iScore2 ;
	volatile int iCPU ;

	CRITICAL_SECTION SPLock ;

} SPLITPOINT, *PSPLITPOINT ;

typedef struct {
	bool			bTurn ;
	bool			bPV ;
	bool			bSngExtension ;
	bool			bEval ;
	bool			bPromotion ;
	bool			bDatabasePosition ;
	bool			bWhiteBreakThrough ;
	bool			bBlackBreakThrough ;
	bool			bOutputBoard ;

	unsigned int	fld[12] ;
	unsigned int	qfld[20] ;

	BITBOARD		bbField[8] ;
	BITBOARD		bbCapture[10] ;

	int				iWhiteSlice ;
	int				iBlackSlice ;

	PVPOSITION		PVPosition ;

	int				iXMoves ;		// Block with (local) MoveGenerate parameters
	int				iXCapture ;
	int				iCapture ;
	int				ilkcd ;
	BITBOARD		bbFrom ;

	int				iDepth ;
	int				iDrawDepth ;

// Position Parameters
	int				iTempo ;

	int				iBlack_Area1_Base ;
	int				iBlack_Area1_Front ;
	int				iBlack_Area3_Base ;
	int				iBlack_Area3_Front ;
	int				iWhite_Area3_Base ;
	int				iWhite_Area3_Front ;
	int				iWhite_Area1_Base ;
	int				iWhite_Area1_Front ;
	int				iBlack_Area2 ;
	int				iWhite_Area2 ;

// Combination Parameters
	BITBOARD		bbCombination0 ;
	BITBOARD		bbCombination1 ;
	int				iCombinationWhiteFrom[66] ;
	int				iCombinationWhiteTo[66] ;
	int				iCombinationBlackFrom[66] ; 
	int				iCombinationBlackTo[66] ; 

	int				matval ;
	int				iScore ;

	int				iAlfa ;
	int				iBeta ;

	HASHCODE			hcode ;		// 64 bit Implementation
	unsigned short int	hIndex ;
	unsigned int		hFlags ;
	unsigned int		hMove ;

	unsigned int	imx_PV ;
	unsigned int	PV[MAX_PV] ;
	unsigned int	PV_Move ;

	int				iwn ;
	int				ibn ;

	int				iWMan ;
	int				iWKing ;
	int				iBMan ;
	int				iBKing ;

	PLOOK			FirstMove ;
	PLOOK			NextMove ;

	BITBOARD		bbKillerMove ;

	int				cm_idx ;
	int				bm_idx ;

	unsigned int	uEvalFlags ;

} TSTACK, *PTSTACK ;


typedef struct {
	int		code ;
	int		from ;
	int		to ;
	bool	bSngMove ;
} MOVE ;


typedef struct {
	int		iFrom ;
	int		iTo ;
	int		iNKill ;
	int		iKill[20] ;
} SMOVE ;


typedef struct {
	bool			bTurn ;
	int				iIndexMove ;
	int				iMoveData[21] ;
	SMOVE			smove ;
	BITBOARD		bbField[8] ;
	LOGDATA			LogData[TREESTACK_SIZE] ;
	CString			csMove ;
} GAMEHISTORY ;

		
typedef struct {
	int		ipfrom ;
	int		ipto ;
	int		ipmouse ;
	int		ipsel ;
} POS, *PPOS ;


typedef struct {
	bool	bTurn ;
	bool	bPVTurn ;
	int		imx ;
	int iPVScore ;
	BITBOARD bbField[8] ;
	MOVE	move[MAX_PV] ;
} PV, *PPV ; 


typedef struct {
	CString	csOutputMessage ;
	int		iMessageSource ;
	int		iMessageID ;
} OUTPUTSTACK ;


typedef struct {

	volatile bool	bStop ;
	volatile bool	bRunning ;
	volatile bool	bIdle ;
	volatile bool	bWorkIsWaiting ;

	int				iPPActiveSplitPoints ;

	PSPLITPOINT		pSplitPoint ;

} SEARCHTHREAD, *PSEARCHTHREAD ;


typedef struct {

	int				iColor ;

	int				iNWhiteMan ;
	int				iNWhiteKing ;
	int				iNBlackMan ;
	int				iNBlackKing ;

	int				iWhiteSlice ;
	int				iBlackSlice ;

	unsigned char	iBoard[93] ;

} DBGENTHREADDATA, *PDBGENTHREADDATA ;

typedef struct {

	bool			bTurn ;
	bool			bDatabasePosition ;
	bool			bZeroWindowSearch ;
	bool			bPV ;

	bool			bLMR ;
	
	int				iSearchThreadID ;

	//SearchThread data
	int				iAlfa ;
	int				iBeta ;
	int				iDepth ;
	int				iDepth0 ;

	int				iMaxDepth ;
	__int64			iTotalDepth	;
	__int64			iDepthNodes ;

	int				iTreeStackDepth ;
	int				iScore ;

	unsigned int	dTimeSearch ;
	struct timeb	SearchThreadStartTime ;
	float			fStartSearchTime ;
	float			fElapsedTime ;

	int				iBoard[51] ;
	BITBOARD		bbField[8] ;
	BBPOSITION		Position[256] ;
	
	int				ipbx ;
	int				iklmx ;

	int				ply	;

	ROOT			Root[NROOT] ;
	int				iRoot ;

	int				am_idx ;	// Analysis Move Index
	int				bm_idx ;	// Best Move Index
	int				cm_idx ;	// Current Move Index
	int				dm_idx ;

	int				iSearchNID ;

	PV				pv ;

	int				score0 ;
	int				score ;
	int				time ;
	__int64			isc[64] ;

	// HashTable Paramaters
	unsigned int	iHT_Entry ; 
	unsigned int	iHT_Index ;
	unsigned int	iHT_Mask ; 

	unsigned int*	pHistoryTable ;
	PEHAREC			pEvalHashTable ;
	PHAREC			pHashTable ;
	PHARECPP		pHashTablePP ;
	PLOOK			pMoveStack ;
	PTSTACK			pTreeStack ;

} SEARCHTHREADDATA, *PSEARCHTHREADDATA ;


typedef struct {

	int		iThreadTask ;
//	HANDLE	hEventStart ;
//	HANDLE	hEventStop ;

} THREADDATA, *PTHREADDATA ;


typedef struct {

	bool	bTurn ;
	bool	bBoardDisplayTurn ;
	bool	bDatabasePosition ;

	bool	bCWhite ;
	bool	bCBlack ;
	bool	bComp ;
	bool	bMove ;
	bool	bGameMove ;
	bool	bSearch ;
	bool	bThink ;

	bool	bBook ;
	bool	bDBEnd ;
	bool	bQSearch ;
	bool	bFHR ;
	bool	bHistory ;
	bool	bSngEx ;
	bool	bLearnDatabase ;
	bool	bHashWrite ;
	bool	bEvalHashWrite ;
	bool	bMinMax ;
	bool	bHashReadETC ;
	bool	bMCP ;
	bool	bSelectiveSearch ;
	bool	bLMR ;
	bool	bIID ;

	bool	bLearnRead ;
	bool	bLearnWrite ;

	bool	bBrain ;

	bool	bBook_Active ;
	bool	bBook_Open ;
	bool	bBook_Use ;

	bool	bProfile ;
	bool	bLogging ;
	bool	bEvalOutput ;

	int		iBookMode ;
	int		iLogMode ;
	
	int				iEngineID ;
	CString			csEngineName ;
	CString			csEngineAuthor ;
	int				iEngineMode ;
	unsigned int	iEngineModeMask ;

	int		iAnalysisSearchMode ;
	int		iAnalysisThreshold ;
	int		iAnalysisSearchValue ;

	int		iNormalSearchMode ;
	int		iNormalSearchValue ;

	int		iGenerateDepth ;		// Depth for BreakThroughTable Generate

	int					iSearchMode ;
	int					iStartSearchDepth ;
	unsigned int		iSearchValue ;

	int		iOpponents ;

	int		iBlitzTime ;
	int		iBlitzTimeLeft ;
	int		iBlitzDeltaTime ;
	int		iBlitzGameMoves ;
	int		iBlitzGameWhiteMovesLeft ;
	int		iBlitzGameBlackMovesLeft ;
	int		iBlitzGameWhiteTimeUsed ;
	int		iBlitzGameBlackTimeUsed ;

	int		iEngineTimeLeft ;

	int		iBlitzMove ;

	bool			bPanicTime ;
	unsigned int	dTimeSearch ;
	unsigned int	dEngineSearchTime ;
	float			fEngineSearchTime ;
	struct timeb	EngineSearchStartTime ;
	struct timeb	EngineSearchFinishTime ;

	int				iBoard[51] ;

	BITBOARD		bbField[8] ;
	BITBOARD		bbFieldDisplay[8] ;
	BBPOSITION		Position[256] ;
	DEBUGDATA		DebugData ;
	
	int		iNMoves ;
	int		iklmx ;

	int		ply	;

	ROOT			Root[NROOT] ;
	ROOTDEPTHINFO	RootDepthInfo[SEARCH_MAXSEARCHDEPTH + 1] ;

	int		am_idx ;	// Analysis Move Index
	int		bm_idx ;	// Best Move Index
	int		cm_idx ;	// Current Move Index
	int		dm_idx ;

	int		iIndexAnalysisMove ;
	int		iIndexBestMove ;
	int		iScoreAnalysisMove ;
	int		iScoreBestMove ;
	CString csAnalysisMove ;
	CString csBestMove ;

	PV		pv ;

	int		score0 ;
	int		score ;
	int		time ;
	__int64	isc[64] ;

	// HashTable
	int					iHashTableSize ;

	// Endgame DB
	int					iDBMaxPiece ;
	int					iDBCacheSize ; 

	// SearchThread Data
	bool				bParallel ;
	bool				bSharedHistory ;
	bool				bSharedHashTable ;
	bool				bSharedEvalHashTable ;
	bool				bCountClear ;
	bool				bPPSearch[SEARCHTHREAD_MAXTHREAD] ;

	bool				bPPAbortSearch ;
	bool				bPPAllThreadsShouldExit ;
	bool				bPPIdle ;

	int					iNCore ;
	int					iNSearchThreadMax ;

	int					iSPOutput ;
	int					iDebugMode ;
	int					iDebugOutput ;

	int					iPPProcessActive ;
	int					iPPMode ;
	int					iPPSubSearchMode ;
	int					iPPOutput ;
	int					iPPMasterThread ;
	int					iPPProcessStatus[SEARCHTHREAD_MAXTHREAD] ;
	int					iPPSearchThreadStatus[SEARCHTHREAD_MAXTHREAD] ;

	int					iPPMaxThreadsPerSplitPoint ;
	int					iPPMaxActiveSplitPoints ;
	int					iPPMinimumSplitBranch ;
	int					iPPMinimumSplitDepth ;
	int					iPPNSplitpoint ;

	// Search Parameters
	int					iPVSplit_SplitDepth ;
	int					iMCP_DepthReductionFactor ;
	int					iMCP_Child_Search ;
	int					iMoveGenMode ;
	int					iMove ;
	int					iMoveIndex ;
	int					iMoveData[32] ;

	int					iSingularExtensionDepth ;

	CString				csSearchPV ;
	
	HANDLE				hEventStart[SEARCHTHREAD_MAXTHREAD] ;
	HANDLE				hEventStop[SEARCHTHREAD_MAXTHREAD] ;
	HANDLE				hSitIdleEvent[SEARCHTHREAD_MAXTHREAD] ;
	HANDLE				hEventThreadReady ;
	HANDLE				hEventSearchStop ;

	CMutex				SendEngineMessageLock ;

	THREADDATA			ThreadData[SEARCHTHREAD_MAXTHREAD] ;
	SEARCHTHREAD		SearchThread[SEARCHTHREAD_MAXTHREAD] ;
	SEARCHTHREADDATA	SearchThreadData[SEARCHTHREAD_MAXTHREAD] ;
	SPLITPOINT			SplitPoint[SEARCHTHREAD_MAXTHREAD][PP_MAXACTIVESPLITPOINTS] ;

	ENDGAMEDBGEN		EndGameDBGen ;
	DXPMATCH			DXPMatch ;

	// Output Data
	int				iOutputStackRead ;
	int				iOutputStackWrite ;
	OUTPUTSTACK		OutputStack[ENGINE_OUTPUT_STACK_SIZE] ;

	// GameHistory Data
	int				iGameHistoryActualMove ;
	int				iGameHistoryMaxMove ;
	GAMEHISTORY		GameHistory[GAMEHISTORY_MAXSIZE] ;
	
	// HashTable Paramaters
	unsigned int iHT_Entry ; 
	unsigned int iHT_Index ;
	unsigned int iHT_Mask ; 

	unsigned int*	pHistoryTable ;
	PEHAREC			pEvalHashTable ;
	PHAREC			pHashTable ;
	PHARECPP		pHashTablePP ;
	PLOOK			pMoveStack ;
	PTSTACK			pTreeStack ;

	FILE*			pDamageLog ;

} ENGINEDATA, *PENGINEDATA ;


inline int DRAUGHTS_PHASE(PTSTACK pTDepth)
{ return ( ( pTDepth->iwn + pTDepth->ibn - 1 ) / 10 ) ; }


#endif	// _CENGINEDEFINITIONS_H_
